/**
 * Module dependencies
 */

const { password } = require('../utils/prompt')
const encrypt = require('../utils/encrypt')
const decrypt = require('../utils/decrypt')
const File = require('../commands/file')
const mkdir = require('co-mkdirp')
const Get = require('./get')
const path = require('path')
const fs = require('co-fs')
const cwd = process.cwd()
const co = require('co')

/**
 * Export `create`
 */

module.exports = co.wrap(create)

/**
 * Initialize `create`
 */

function * create ({ root = cwd }) {
  const statepath = path.resolve(root, 'thimble.state')
  const relative = path.relative(cwd, statepath)

  const exists = yield fs.exists(statepath)
  if (exists) return relative

  const pass = yield password(`
    \n
    Let's create a state file at: ./${relative}

    This file will store all data you and your teammates enter
    during the life of your app. It will be encrypted so you
    can store secrets like API tokens and other private application
    state.

    Please create a password to encrypt this file:
  `)

  const again = yield password(`
    And one more time?

    Please save this password somewhere or
    you will not be able to access your state.

    Password (again):
  `)

  if (pass !== again) {
    throw new Error(`Passwords don't match!`)
  }

  const encrypted = encrypt('{}', pass)
  yield mkdir(path.dirname(statepath))
  yield fs.writeFile(statepath, encrypted)
  return relative
}
