/**
 * Module dependencies
 */

const install = require('npminstall').installLocal
const assign = require('object-assign')
const npa = require('npm-package-arg')
const copy = require('../utils/copy')
const rm = require('../utils/rimraf')
const glob = require('../utils/glob')
const YAML = require('js-yaml')
const List = require('./list')
const isArray = Array.isArray
const path = require('path')
const fs = require('co-fs')
const cwd = process.cwd()
const co = require('co')

/**
 * Export `Get`
 */

module.exports = co.wrap(Get)

/**
 * Mock console for `npminstall`
 */

const cons = {
  error:() => {},
  log: () => {},
  warn:() => {},
  info:() => {}
}

/**
 * Initialize `Get`
 */

function * Get ({ remotes = [], root = cwd, path: module_path = 'node_modules' }) {
  remotes = isArray(remotes) ? remotes : [remotes]
  const packages = []
  const paths = []

  for (let remote of remotes) {
    let pkg = npa(remote)
    let src = path.resolve(root, pkg.name || pkg.spec)
    let dirname = path.basename(src)
    let type = pkg.type

    if (type === 'local') {
      paths.push(path.resolve(root, remote))
    } else {
      paths.push(path.resolve(module_path, dirname))
      packages.push({
        version: pkg.spec || 'latest',
        name: pkg.name,
        type: pkg.type
      })
    }
  }

  // install remotes
  if (packages.length) {
    yield install({
      root: root,
      pkgs: packages,
      console: cons
    })
  }

  return paths
}
