/**
 * Module Dependencies
 */

let Transform = require('./transform')
let Mergetool = require('./mergetool')
let Merge = require('./merge')
let Write = require('./write')
let Read = require('./read')
let path = require('path')

/**
 * Export `Add`
 */

module.exports = Add

/**
 * Add(from, to, answers)
 *
 * - High-level of READ -> TRANSFORM -> WRITE
 */

function Add (params) {
  let TRANSFORM = Transform(params)
  let MERGETOOL = Mergetool(params)
  let { root, directory } = params
  let WRITE = Write(params)
  let MERGE = Merge(params)
  let READ = Read(params)

  return function * Add (from, to = './', answers = {}) {
    from = path.resolve(directory, from)
    to = path.resolve(root, to)
    let files = yield READ(from)
    files = yield TRANSFORM(files, answers)
    let conflicts = yield WRITE(files, to)
    if (!conflicts.length) return
    let unmerged = yield MERGE(conflicts, to)
    if (!unmerged.length) return
    let merged = yield MERGETOOL(unmerged)
    return merged
  }
}
