/**
 * Module dependencies
 */

const Vinyl = require('vinyl')
const path = require('path')
const poss = require('poss')
const fs = require('co-fs')

/**
 * Export `File`
 */

module.exports = File

/**
 * Initialize `File`
 */

function File ({ directory }) {
  return function * file (filepath, base) {
    const isdir = filepath[filepath.length - 1] === '/'
    const cwd = base ? path.resolve(directory, base) : directory
    filepath = path.resolve(cwd, filepath)
    filepath = isdir ? filepath + '/' : filepath

    const [err, stat] = yield poss(fs.stat(filepath))
    const file = { cwd, path: filepath }

    // handle files that haven't been created yet
    if (stat) file.stat = stat
    else return new Vinyl(file)

    // if it's a directory just return the vinyl file
    if (file.stat.isDirectory()) return new Vinyl(file)
    // otherwise if it's a file, get it's contents
    file.contents = yield fs.readFile(file.path)
    return new Vinyl(file)
  }
}
