// HELPS WITH A MERGE AND MERGE CONFLICTS
//
// - DIFF(...)
// - PROMPT(...)
// - RENAME(...)

/**
 * Commands
 */

let Overwrite = require('./overwrite')
let indent = require('indent-string')
let Diff = require('../utils/diff')
let Prompt = require('./prompt')
let Rename = require('./rename')
let path = require('path')

/**
 * Export `Mergetool`
 */

module.exports = Mergetool

/**
 * Initialize `Mergetool`
 */

function Mergetool (params) {
  let OVERWRITE = Overwrite(params)
  let PROMPT = Prompt(params)
  let RENAME = Rename(params)
  let { root } = params

  return function * mergetool (conflicts) {
    for (let conflict of conflicts) {
      let [ left, right ] = conflict
      let diff = Diff(left, right, params.root)
      let strategy = null

      do {
        strategy = yield PROMPT(`
        **MERGE CONFLICT**

        Right now there are two files that need to be manually merged.
        Here's the difference between these two files:


        ${indent(diff, 8).trim()}

        And here are the options you have to resolve this conflict.

          - [R] Rename NEW FILE to no longer conflict with the ORIGINAL
          - [O] Overwrite ORIGINAL with NEW FILE
          - [K] Keep ORIGINAL and ignore NEW FILE

        Which option would you like to use? [R|O|K]:
      `)
      } while (strategy === null || !/[ROK]/i.test(strategy))

      // execute the strategy
      switch (strategy.toUpperCase()) {
        case 'R': return yield OVERWRITE(yield RENAME(right), left.dirname)
        case 'O': return yield OVERWRITE(right, left.dirname)
        case 'K': return
      }
    }
  }
}
