/**
 * Module dependencies
 */

const Indent = require('indent-stream')
const spawn = require('cross-spawn')
const parse = require('spawn-args')
const chalk = require('chalk')

/**
 * Export `Run`
 */

module.exports = Run

/**
 * Initialize `Run`
 */

function Run ({ root, plan }) {
  return function * run (cmd) {
    return yield function (fn) {
      let args = typeof cmd === 'string' ? parse(cmd) : cmd
      let command = args.join(' ')

      console.error()

      if (plan) {
        console.error(chalk.blue(`  WOULD RUN `) + command)
        return fn()
      } else {
        console.error(chalk.blue(`  RUN `) + command)
      }

      // redirect both to stderr
      let cp = spawn(args[0], args.slice(1), { cwd: root })
      let indent = Indent(chalk.blue(`   |  `))

      cp.stderr.pipe(indent)
      cp.stdout.pipe(indent)

      indent.on('data', (data) => {
        process.stderr.write(log(data.toString()))
      })

      cp.on('error', fn)
      cp.on('exit', (code) => {
        return code > 0
          ? fn(new Error(`${command} exited with the error code ${code}`))
          : fn()
      })
    }
  }
}

function log (log) {
  return chalk.gray(log)
}

/**
 * Execute `cmd`.
 */

function exec (cmd, opts){
  return function(done) {
    cp.exec(cmd, opts, function(err, stdout, stderr){
      done(err, { stdout, stderr })
    })
  }
}
