/**
 * Module dependencies
 */

const stringify = require('../utils/stringify')
const parse = require('../utils/parse')
const isArray = Array.isArray

/**
 * Export `Transform`
 */

module.exports = Transform

/**
 * Initialize `Transform`
 */

function Transform (params) {
  return function * transform (files, map) {
    if (arguments.length === 1) map = (file) => file
    else if ('object' === typeof map) map = Map(map)
    files = isArray(files) ? files : [ files ]
    return yield files.map(map)
  }
}

/**
 * Default map
 */

function Map (locals = {}) {
  return function map (file) {
    let ast = parse(file)

    if (ast.path) {
      file.path = stringify(ast.path, locals)
    }

    if (ast.contents) {
      file.contents = new Buffer(stringify(ast.contents, locals))
    }

    return file
  }
}
