/**
 * Module dependencies
 */

const mkdir = require('co-mkdirp')
const File = require('./file')
const isArray = Array.isArray
const poss = require('poss')
const path = require('path')
const fs = require('co-fs')

/**
 * Export `Write`
 */

module.exports = Write

/**
 * Initialize `Write`
 */

function Write (params) {
  const { root, overwrite = false } = params
  const FILE = File(params)

  return function * write (files, content = '') {
    files = isArray(files) ? files : [files]

    const flag = overwrite ? 'w' : 'wx'
    let conflicts = []

    yield files.map(function * (file) {
      if (typeof file === 'string') {
        const isdir = file[file.length - 1] === '/'
        const outpath = path.resolve(root, file)
        if (isdir) return yield mkdir(outpath)
        const dirname = path.dirname(outpath)
        yield mkdir(dirname)
        const conflict = yield writefile(outpath, content, { flag })
        const vinyl = yield FILE(outpath)
        conflicts = conflict ? conflicts.concat(vinyl) : conflicts
        return
      }

      const outpath = path.resolve(root, file.relative)
      const dirname = path.dirname(outpath)

      // mkdir
      yield mkdir(dirname)

      // skip directories
      if (file.isDirectory()) return

      // write out or if there's a conflict add to array and return
      let conflict = yield writefile(outpath, file.contents, { flag })
      conflicts = conflict ? conflicts.concat(file) : conflicts
    })

    return conflicts.length ? conflicts : []
  }
}

function * writefile (path, contents, flag) {
  const [ err ] = yield poss(fs.writeFile(path, contents, flag))
  if (err && err.code === 'EEXIST') return path
  else if (err) throw err
  return null
}
