/**
 * Module dependencies
 */

const assign = require('object-assign')
const glob = require('../utils/glob')
const uniq = require('lodash.uniq')
const YAML = require('js-yaml')
const path = require('path')
const fs = require('co-fs')

/**
 * Export `resolve`
 */

module.exports = resolve

/**
 * Initialize `resolve`
 */

function * resolve (dirs) {
  dirs = uniq(dirs)
  let commands = []
  let paths = {}

  for (let dir of dirs) {
    let files = yield glob('thimble.yml', { cwd: dir, matchBase: true })
    for (let file of files) {
      let fullpath = path.resolve(dir, file)
      let yaml = yield fs.readFile(fullpath)
      let dirname = path.dirname(fullpath)
      let json = YAML.safeLoad(yaml)
      for (let command in json) {
        if (!json[command].path) continue
        let cp = path.resolve(dirname, json[command].path)
        if (paths[cp]) continue
        paths[cp] = true
        commands.push(assign({}, json[command], {
          directory: path.basename(dirname),
          command: command,
          path: cp
        }))
      }
    }
  }

  return commands
}
